unit MC_Form;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  ExtCtrls, CommCtrl, ComCtrls, Menus, Registry, MMSystem, Ex_Controls,
  pop3cli, Wait, MC_Mail, MC_Options;

type
  TCheckerForm = class(TForm)
    MainMenu: TMainMenu;
    FileMenu: TMenuItem;
    FileOpenItem: TMenuItem;
    FileDeleteItem: TMenuItem;
    FileN1Item: TMenuItem;
    FilePropertiesItem: TMenuItem;
    FileN2Item: TMenuItem;
    FileExitItem: TMenuItem;
    ViewMenu: TMenuItem;
    ViewStatusBarItem: TMenuItem;
    ViewN1Item: TMenuItem;
    ViewCharSetItem: TMenuItem;
    ViewDOSItem: TMenuItem;
    ViewWINItem: TMenuItem;
    ViewKOIItem: TMenuItem;
    ViewN3Item: TMenuItem;
    FileCheckItem: TMenuItem;
    ViewOptionsItem: TMenuItem;
    HelpMenu: TMenuItem;
    HelpTopicsItem: TMenuItem;
    HelpN1Item: TMenuItem;
    HelpAboutItem: TMenuItem;
    PopupMenu: TPopupMenu;
    PopupN0Item: TMenuItem;
    PopupOpenItem: TMenuItem;
    PopupDeleteItem: TMenuItem;
    PopupN1Item: TMenuItem;
    PopupPropertiesItem: TMenuItem;
    TrayMenu: TPopupMenu;
    TrayAboutItem: TMenuItem;
    TrayN1Item: TMenuItem;
    TrayCheckerItem: TMenuItem;
    TrayOptionsItem: TMenuItem;
    TrayN2Item: TMenuItem;
    TrayCheckItem: TMenuItem;
    TrayN3Item: TMenuItem;
    TrayExitItem: TMenuItem;
    SmallImages: TImageList;
    Timer: TTimer;
    FormPlacement: TFormPlacement;
    FormHook: TFormHook;
    AppHook: TAppHook;
    TrayIcon: TTrayIcon;
    AboutDialog: TAboutDialog;
    POP3: TPop3Client;
    POP3Wait: TWait;
    MailView: TListView;
    StatusBar: TStatusBar;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormHide(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FileMenuClick(Sender: TObject);
    procedure FileOpenItemClick(Sender: TObject);
    procedure FileDeleteItemClick(Sender: TObject);
    procedure FilePropertiesItemClick(Sender: TObject);
    procedure FileCheckItemClick(Sender: TObject);
    procedure FileExitItemClick(Sender: TObject);
    procedure ViewStatusBarItemClick(Sender: TObject);
    procedure ViewCharSetItemClick(Sender: TObject);
    procedure ViewOptionsItemClick(Sender: TObject);
    procedure HelpTopicsItemClick(Sender: TObject);
    procedure HelpAboutItemClick(Sender: TObject);
    procedure PopupMenuPopup(Sender: TObject);
    procedure TrayMenuPopup(Sender: TObject);
    procedure TrayCheckerItemClick(Sender: TObject);
    procedure TimerTimer(Sender: TObject);
    procedure PlacementLoad(Sender: TObject; IniFile: TRegIniFile);
    procedure PlacementSave(Sender: TObject; IniFile: TRegIniFile);
    procedure FormHookEnter(var Message: TMessage; var Handled: Boolean);
    procedure AppHookAppHint(Sender: TObject);
    procedure AppHookEnter(var Message: TMessage; var Handled: Boolean);
    procedure POP3MessageBegin(Sender: TObject);
    procedure POP3MessageLine(Sender: TObject);
    procedure MailViewDblClick(Sender: TObject);
    procedure MailViewKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
  private
    FMailBox: TMailBox;
    FOptions: TOptions;
    FCharSet: Integer;
    FPOPBuffer: TStringList;
    FPOPCount: Integer;
    FPOPHint: string;
    function GetMailBox: TMailBox;
    function GetMessageIndex: Integer;
    function GetOptions: TOptions;
    function GetPOPBuffer: TStringList;
    procedure SetCharSet(Value: Integer);
    procedure SetIcon(Value: Integer);
    procedure SetHint(Value: string);
  protected
    function POPExec(Commands: array of TPop3Method): Boolean;
    function POPStart: Boolean;
    function POPStop: Boolean;
    procedure UpdateOptions;
    procedure UpdateView;
    property POPBuffer: TStringList read GetPOPBuffer;
  public
    function CheckMail: Boolean;
    function DeleteMessage(Index: Integer): Boolean;
    function OpenMessage(Index: Integer): Boolean;
    property CharSet: Integer read FCharSet write SetCharSet;
    property MailBox: TMailBox read GetMailBox;
    property MessageIndex: Integer read GetMessageIndex;
    property Options: TOptions read GetOptions;
  end;

var
  CheckerForm: TCheckerForm;

implementation

uses
  MC_Source, MC_CharSet, MC_Properties, MC_Cursor, MC_Consts;

{$R *.DFM}
{$R *.RES}

{ TCheckerForm }

function TCheckerForm.GetMailBox: TMailBox;
begin
  if FMailBox = nil then FMailBox := TMailBox.Create;
  Result := FMailBox;
end;

function TCheckerForm.GetMessageIndex: Integer;
begin
  with MailView do Result := Items.IndexOf(Selected);
end;

function TCheckerForm.GetOptions: TOptions;
begin
  if FOptions = nil then FOptions := Toptions.Create;
  Result := FOptions;
end;

function TCheckerForm.GetPOPBuffer: TStringList;
begin
  if FPOPBuffer = nil then FPOPBuffer := TStringList.Create;
  Result := FPOPBuffer;
end;

procedure TCheckerForm.SetCharSet(Value: Integer);
begin
  FCharSet := Value;
  case Value of
    CS_DOS: ViewDOSItem.Checked := True;
    CS_WIN: ViewWINItem.Checked := True;
    CS_KOI: ViewKOIItem.Checked := True;
  end;
  UpdateView;
end;

procedure TCheckerForm.SetIcon(Value: Integer);

  function LoadIcon(const Name: string): HICON;
  begin
    Result := LoadImage(HInstance, PChar(Name), IMAGE_ICON, 0, 0, LR_LOADREALSIZE);
  end;

const
  Icons: array[0..9] of string = (
    'MBOX0', 'MBOX1', 'MBOX2', 'MBOX3', 'MBOX4', 'MBOX5', 'MBOX6',
    'MBOX7', 'MBOX8', 'MBOX9');
begin
  if Value < 0 then TrayIcon.Icon.Handle := LoadIcon('MBOX00') else
  if Value > 9 then TrayIcon.Icon.Handle := LoadIcon('MBOX99')
  else TrayIcon.Icon.Handle := LoadIcon(PChar(Icons[Value]));
end;

procedure TCheckerForm.SetHint(Value: string);
begin
  if Length(Value) = 0 then
    FPOPHint := Format(LoadStr(SMessages), [MailBox.Count]) else
    FPOPHint := Value;
  TrayIcon.Hint := FPOPHint;
  StatusBar.SimpleText := FPOPHint;
  StatusBar.Update;
end;

function TCheckerForm.POPExec(Commands: array of TPop3Method): Boolean;
var
  I: Integer;
begin
  Result := False;
  try
    for I := Low(Commands) to High(Commands) do if not Commands[I] then Exit;
    Result := True;
  except
    { Ignore exceptions }
  end;
end;

function TCheckerForm.POPStart: Boolean;
begin
  Result := False;
  SetHint(LoadStr(SConnecting));
  if not POPExec([POP3.Connect]) then
  begin
    SetHint(LoadStr(SConnectError));
    Exit;
  end;
  SetHint(LoadStr(SAuthorisation));
  if not POPExec([POP3.User, POP3.Pass]) then
  begin
    POPExec([POP3.Quit]);
    SetHint(LoadStr(SAuthoriseError));
    Exit;
  end;
  Result := True;
end;

function TCheckerForm.POPStop: Boolean;
begin
  Result := POPExec([POP3.Quit]);
end;

procedure TCheckerForm.UpdateOptions;
begin
  with Options do
  begin
    POP3.Port := Port;
    POP3.Host := Host;
    POP3.UserName := UserName;
    POP3.Password := Password;
    Timer.Interval := CheckInterval * 60 * 1000;
    Timer.Enabled := Checking;
  end;
end;

procedure TCheckerForm.UpdateView;
var
  I, C: Integer;

  procedure SetCaption(Item: Integer; const Value: string);
  begin
    with MailView.Items[Item] do
      if CompareText(Caption, Value) <> 0 then
      begin
        Caption := Value;
        Inc(C);
      end;
  end;

  procedure SetSubItem(Item, Index: Integer; const Value: string);
  begin
    with MailView.Items[Item].SubItems do
      if CompareText(Strings[Index], Value) <> 0 then
      begin
        Strings[Index] := Value;
        Inc(C);
      end;
  end;

  procedure UpdateItem(Item: Integer);
  var
    R: TRect;
  begin
    with MailView do
    begin
      R := Items[Item].DisplayRect(drBounds);
      InvalidateRect(Handle, @R, False);
    end;
  end;

begin
  if not (csLoading in ComponentState) then
  begin
    with MailView do
    begin
      if Items.Count > MailBox.Count then
      begin
        I := Items.Count;
        while I > MailBox.Count do
        begin
          Dec(I);
          Items.Delete(I);
        end;
      end else
      if Items.Count < MailBox.Count then
      begin
        I := Items.Count;
        while I < MailBox.Count do
        begin
          Inc(I);
          with Items.Add do
          begin
            SubItems.Add(''); { [0] }
            SubItems.Add(''); { [1] }
          end;
        end;
      end;
      for I := 0 to MailBox.Count - 1 do
        with MailBox[I] do
        begin
          C := 0;
          SetCaption(I, ChangeCharSet(From, CharSet, CS_WIN));
          SetSubItem(I, 0, ChangeCharset(Subject, CharSet, CS_WIN));
          SetSubItem(I, 1, ChangeCharset(Date, CharSet, CS_WIN));
          if C <> 0 then UpdateItem(I);
        end;
    end;
  end;
end;

function TCheckerForm.CheckMail: Boolean;
var
  I, Count, New: Integer;
  Text: string;
begin
  Result := False;
  if FPOPCount = 0 then
  begin
    Inc(FPOPCount);
    SetCursor(crHourGlass);
    try
      Count := -1;
      SetIcon(-1);
      if POPStart then
      try
        SetHint(LoadStr(SCheckingMail));
        if POPExec([POP3.Stat]) then
        begin
          Count := MailBox.Count;
          MailBox.Clear;
          try
            New := POP3.MsgCount;
            for I := 1 to New do
            begin
              POP3.MsgNum := I;
              if POPExec([POP3.TOP]) then MailBox.Add(POPBuffer.Text);
            end;
          finally
            UpdateView;
          end;
          SetHint('');
          SetIcon(MailBox.Count);
          Result := True;
        end else
        begin
          SetHint(LoadStr(SCheckError));
        end;
      finally
        POPStop;
      end;
      if Count < MailBox.Count then
      begin
        if Options.MessageBeep then
        begin
          if Count <> -1 then sndPlaySound('MailBeep', SND_ASYNC);
        end;
        if Options.MessageBox then
        begin
          if Count <> -1 then
            Text := LoadStr(SNewMail)
          else
            Text := TrayIcon.Hint + '.';
          with Application do
            MessageBox(PChar(Text), PChar(Title), MB_ICONWARNING or MB_OK);
        end;
      end;
    finally
      SetCursor(crDefault);
      Dec(FPOPCount);
    end;
  end;
end;

function TCheckerForm.DeleteMessage(Index: Integer): Boolean;
var
  Count: Integer;
  Text: string;

  function DeleteQuery: Boolean;
  const
    Flags = MB_ICONQUESTION or MB_YESNO or MB_DEFBUTTON2;
  var
    Query: string;
  begin
    Query := LoadStr(SDeleteQuery);
    with Application do
      Result := MessageBox(PChar(Query), PChar(Title), Flags) = ID_YES;
  end;

begin
  Result := False;
  if (FPOPCount = 0) and (Index <> -1) and DeleteQuery then
  begin
    Inc(FPOPCount);
    SetCursor(crHourGlass);
    try
      Count := -1;
      SetIcon(-1);
      if POPStart then
      try
        SetHint(LoadStr(SDeletingMessage));
        POP3.MsgNum := Index + 1;
        if POPExec([POP3.Dele]) then
        begin
          Count := MailBox.Count;
          MailBox.Delete(Index);
          UpdateView;
          SetHint('');
          SetIcon(MailBox.Count);
          Result := True;
        end else
        begin
          SetHint(LoadStr(SDeleteError));
        end;
      finally
        POPStop;
      end;
      if Options.MessageBox and (Count = -1) then
      begin
        Text := TrayIcon.Hint + '.';
        MessageBox(Handle, PChar(Text), PChar(Caption), MB_ICONWARNING);
      end;
    finally
      SetCursor(crDefault);
      Dec(FPOPCount);
    end;
  end;
end;

function TCheckerForm.OpenMessage(Index: Integer): Boolean;
var
  Count: Integer;
  Text: string;
begin
  Result := False;
  if (FPOPCount = 0) and (Index <> -1) then
  begin
    Inc(FPOPCount);
    SetCursor(crHourGlass);
    try
      Count := -1;
      SetIcon(-1);
      if POPStart then
      try
        SetHint(LoadStr(SLoadingMessage));
        POP3.MsgNum := Index + 1;
        if POPExec([POP3.Retr]) then
        begin
          Count := MailBox.Count;
          ShowMessageSource(POPBuffer.Text);
          SetHint('');
          SetIcon(MailBox.Count);
          Result := True;
        end else
        begin
          SetHint(LoadStr(SLoadingError));
        end;
      finally
        POPStop;
      end;
      if Options.MessageBox and (Count = -1) then
      begin
        Text := TrayIcon.Hint + '.';
        MessageBox(Handle, PChar(Text), PChar(Caption), MB_ICONWARNING);
      end;
    finally
      SetCursor(crDefault);
      Dec(FPOPCount);
    end;
  end;
end;

procedure TCheckerForm.FormCreate(Sender: TObject);
const
  LVM_SETEXTSTYLE      = $1000 + 54;
  LVS_EX_FULLROWSELECT = $00000020;
begin
  SendMessage(MailView.Handle, LVM_SETEXTSTYLE, 0, LVS_EX_FULLROWSELECT);
  SmallImages.ResourceLoad(rtBitmap, 'MAIL', clOlive);
  UpdateOptions;
  with Options do
  begin
    if not Visible then
    begin
      ShowWindow(Application.Handle, SW_HIDE);
      Application.ShowMainForm := False;
    end;
    if Checking then
    begin
      if Visible then Show;
      CheckMail;
    end;
  end;
end;

procedure TCheckerForm.FormDestroy(Sender: TObject);
begin
  FPOPBuffer.Free;
  FOptions.Free;
  FMailBox.Free;
end;

procedure TCheckerForm.FormHide(Sender: TObject);
begin
  if not (csDestroying in ComponentState) then
  begin
    with Application do
    begin
      Minimize;
      ShowWindow(Handle, SW_HIDE);
    end;
    Options.Visible := False;
  end;
end;

procedure TCheckerForm.FormShow(Sender: TObject);
begin
  if not (csLoading in ComponentState) then
  begin
    with Application do
    begin
      ShowWindow(Handle, SW_SHOW);
      Restore;
    end;
    Options.Visible := True;
  end;
end;

procedure TCheckerForm.FileMenuClick(Sender: TObject);
begin
  FileOpenItem.Enabled := (FPOPCount = 0) and (MessageIndex <> -1);
  FileDeleteItem.Enabled := (FPOPCount = 0) and (MessageIndex <> -1);
  FilePropertiesItem.Enabled := MessageIndex <> -1;
  FileCheckItem.Enabled := FPOPCount = 0;
end;

procedure TCheckerForm.FileOpenItemClick(Sender: TObject);
begin
  OpenMessage(MessageIndex);
end;

procedure TCheckerForm.FileDeleteItemClick(Sender: TObject);
begin
  DeleteMessage(MessageIndex);
end;

procedure TCheckerForm.FilePropertiesItemClick(Sender: TObject);
begin
  if MessageIndex <> -1 then ShowMessageProperties(MailBox[MessageIndex].Text);
end;

procedure TCheckerForm.FileCheckItemClick(Sender: TObject);
var
  Count: Integer;
  Text: string;
begin
  Count := MailBox.Count;
  if CheckMail and (Count >= MailBox.Count) then
  begin
    Text := LoadStr(SNoNewMail);
    with Application do
      MessageBox(PChar(Text), PChar(Title), MB_ICONWARNING or MB_OK);
  end;
end;

procedure TCheckerForm.FileExitItemClick(Sender: TObject);
begin
  Application.Terminate;
end;

procedure TCheckerForm.ViewStatusBarItemClick(Sender: TObject);
begin
  with Sender as TMenuItem do
  begin
    Checked := not Checked;
    StatusBar.Visible := Checked;
  end;
end;

procedure TCheckerForm.ViewCharSetItemClick(Sender: TObject);
begin
  CharSet := (Sender as TMenuItem).Tag;
end;

procedure TCheckerForm.ViewOptionsItemClick(Sender: TObject);
begin
  if ChangeOptions(Options) then
  begin
    UpdateOptions;
    if Options.Checking then CheckMail;
  end;
end;

procedure TCheckerForm.HelpTopicsItemClick(Sender: TObject);
begin
  Application.HelpCommand(HELP_FINDER, 0);
end;

procedure TCheckerForm.HelpAboutItemClick(Sender: TObject);
begin
  AboutDialog.Execute;
end;

procedure TCheckerForm.PopupMenuPopup(Sender: TObject);
begin
  PopupOpenItem.Enabled := (FPOPCount = 0) and (MessageIndex <> -1);
  PopupDeleteItem.Enabled := (FPOPCount = 0) and (MessageIndex <> -1);
  PopupPropertiesItem.Enabled := MessageIndex <> -1;
end;

procedure TCheckerForm.TrayMenuPopup(Sender: TObject);
begin
  TrayCheckItem.Enabled := FPOPCount = 0;
end;

procedure TCheckerForm.TrayCheckerItemClick(Sender: TObject);
begin
  Show;
  with Application do
  begin
    Restore;
    BringToFront;
  end;
end;

procedure TCheckerForm.TimerTimer(Sender: TObject);
begin
  CheckMail;
end;

procedure TCheckerForm.PlacementLoad(Sender: TObject; IniFile: TRegIniFile);
var
  I: Integer;
  Ident: string;
begin
  with IniFile do
  begin
    if not ReadBool('', 'StatusBar', True) then ViewStatusBarItem.Click;
    CharSet := ReadInteger('', 'CharSet', CS_KOI);
    with MailView do
    begin
      HandleNeeded;
      for I := 0 to Columns.Count - 1 do
      begin
        Ident := Format('Column%d', [I]);
        ListView_SetColumnWidth(Handle, I, ReadInteger('', Ident, Columns[I].Width));
      end;
    end;
  end;
end;

procedure TCheckerForm.PlacementSave(Sender: TObject; IniFile: TRegIniFile);
var
  I: Integer;
  Ident: string;
begin
  with IniFile do
  begin
    WriteBool('', 'StatusBar', ViewStatusBarItem.Checked);
    WriteInteger('', 'CharSet', CharSet);
    with MailView do
      for I := 0 to Columns.Count - 1 do
      begin
        Ident := Format('Column%d', [I]);
        WriteInteger('', Ident, Columns[I].Width);
      end;
  end;
end;

procedure TCheckerForm.FormHookEnter(var Message: TMessage; var Handled: Boolean);
begin
  with Message do
    if Msg = WM_CLOSE then
    begin
      Hide;
      Handled := True;
    end;
end;

procedure TCheckerForm.AppHookAppHint(Sender: TObject);
begin
  if Length(Application.Hint) = 0 then
  begin
    StatusBar.SimplePanel := True;
    StatusBar.SimpleText := FPOPHint
  end else
  begin
    StatusBar.SimplePanel := False;
    StatusBar.Panels[0].Text := Application.Hint;
  end;
end;

procedure TCheckerForm.AppHookEnter(var Message: TMessage; var Handled: Boolean);
begin
  with Message do
    if (Msg = WM_SYSCOMMAND) and ((WParam and $FFF0) = SC_CLOSE) then
    begin
      Hide;
      Handled := True;
    end;
end;

procedure TCheckerForm.POP3MessageBegin(Sender: TObject);
begin
  POPBuffer.Clear;
end;

procedure TCheckerForm.POP3MessageLine(Sender: TObject);
begin
  POPBuffer.Add(POP3.LastResponse);
end;

procedure TCheckerForm.MailViewDblClick(Sender: TObject);
begin
  OpenMessage(MessageIndex);
end;

procedure TCheckerForm.MailViewKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  if Key = VK_RETURN then
  begin
    FileMenu.Click;
    FileOpenItem.Click;
    Key := 0;
  end else
  if Key = VK_DELETE then
  begin
    FileMenu.Click;
    FileDeleteItem.Click;
    Key := 0;
  end else
  if Key = VK_F5 then
  begin
    FileCheckItem.Click;
    Key := 0;
  end;
end;

end.

