unit MC_Mail;

interface

uses Classes;

const

{ Message headers }

  SReturnPath = 'Return-Path: ';
  SReceived = 'Received: ';
  STo = 'To: ';
  SNewsgroups = 'Newsgroups: ';
  SFrom = 'From: ';
  SSubject = 'Subject: ';
  SOrganization = 'Organization: ';
  SDate = 'Date: ';
  SMailMessageID = 'MailMessage-ID: ';
  SReplyTo = 'Reply-To: ';
  SSender = 'Sender: ';
  SPrecedence = 'Precedence: ';
  SStatus = 'Status: ';
  SXMozillaStatus = 'X-Mozilla-Status: ';

type

{ TMailMessage }

  TMailMessage = class(TPersistent)
  private
    FText: string;
    function GetDate: string;
    function GetFrom: string;
    function GetMailMessageID: string;
    function GetNewsgroups: string;
    function GetOrganization: string;
    function GetReplyTo: string;
    function GetSender: string;
    function GetSubject: string;
  public
    constructor Create(AText: string);
    procedure Assign(Source: TPersistent); override;
    property Date: string read GetDate;
    property From: string read GetFrom;
    property Newsgroups: string read GetNewsgroups;
    property MailMessageID: string read GetMailMessageID;
    property Organization: string read GetOrganization;
    property ReplyTo: string read GetReplyTo;
    property Sender: string read GetSender;
    property Subject: string read GetSubject;
    property Text: string read FText write FText;
  end;

{ TMailBox }

  TMailBox = class(TPersistent)
  private
    FMessages: TList;
    function GetCount: Integer;
    function GetMailMessage(Index: Integer): TMailMessage;
  public
    constructor Create;
    destructor Destroy; override;
    procedure Add(const Text: string);
    procedure Assign(Source: TPersistent); override;
    procedure Clear;
    procedure Delete(Index: Integer);
    property Count: Integer read GetCount;
    property Message[Index: Integer]: TMailMessage read GetMailMessage; default;
  end;

function ExtractMessageHeader(const Message, Header: string): string;
function ExtractMessageHeaders(const Message: string): string;
function ExtractMessageBody(const Message: string): string;

implementation

function ExtractMessageHeader(const Message, Header: string): string;
var
  P: Integer;
begin
  P := Pos(Header, Message);
  if P <> 0 then
  begin
    P := P + Length(Header);
    SetString(Result, PChar(Message) + P - 1, Length(Message) - P + 1);
    P := Pos(#13#10, Result);
    if P <> 0 then SetLength(Result, P - 1);
  end
  else
    Result := '';
end;

function ExtractMessageHeaders(const Message: string): string;
var
  P: Integer;
begin
  P := Pos(#13#10#13#10, Message);
  if P <> 0 then
    Result := Copy(Message, 1, P - 1) else
    Result := Message;
end;

function ExtractMessageBody(const Message: string): string;
var
  P: Integer;
begin
  P := Pos(#13#10#13#10, Message);
  if P <> 0 then
    Result := Copy(Message, P + 4, Length(Message)) else
    Result := Message;
end;

{ TMailMessage }

constructor TMailMessage.Create(AText: string);
begin
  FText := AText;
end;

procedure TMailMessage.Assign(Source: TPersistent);
var
  SourceMessage: TMailMessage absolute Source;
begin
  if Source is TMailMessage then
  begin
    Text := SourceMessage.Text;
    Exit;
  end;
  inherited Assign(Source);
end;

function TMailMessage.GetDate: string;
begin
  Result := ExtractMessageHeader(FText, SDate);
end;

function TMailMessage.GetFrom: string;
begin
  Result := ExtractMessageHeader(FText, SFrom);
end;

function TMailMessage.GetMailMessageID: string;
begin
  Result := ExtractMessageHeader(FText, SMailMessageID);
end;

function TMailMessage.GetNewsgroups: string;
begin
  Result := ExtractMessageHeader(FText, SNewsgroups);
end;

function TMailMessage.GetOrganization: string;
begin
  Result := ExtractMessageHeader(FText, SOrganization);
end;

function TMailMessage.GetReplyTo: string;
begin
  Result := ExtractMessageHeader(FText, SReplyTo);
end;

function TMailMessage.GetSender: string;
begin
  Result := ExtractMessageHeader(FText, SSender);
end;

function TMailMessage.GetSubject: string;
begin
  Result := ExtractMessageHeader(FText, SSubject);
end;

{ TMailBox }

constructor TMailBox.Create;
begin
  FMessages := TList.Create;
end;

destructor TMailBox.Destroy;
begin
  Clear;
  FMessages.Free;
end;

function TMailBox.GetCount: Integer;
begin
  Result := FMessages.Count;
end;

function TMailBox.GetMailMessage(Index: Integer): TMailMessage;
begin
  Result := FMessages[Index];
end;

procedure TMailBox.Add(const Text: string);
begin
  FMessages.Add(TMailMessage.Create(Text));
end;

procedure TMailBox.Assign(Source: TPersistent);
var
  SourceBox: TMailBox absolute Source;
  I: Integer;
begin
  if Source is TMailBox then
  begin
    Clear;
    for I := 0 to SourceBox.Count - 1 do Add(SourceBox[I].Text);
    Exit;
  end;
  inherited Assign(Source);
end;

procedure TMailBox.Clear;
var
  I: Integer;
begin
  I := Count;
  while I > 0 do
  begin
    Dec(I);
    Delete(I);
  end;
end;

procedure TMailBox.Delete(Index: Integer);
begin
  Message[Index].Free;
  FMessages.Delete(Index);
end;

end.
