unit MC_Options;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, 
  StdCtrls, Registry;

type

{ TOptions }

  TOptions = class(TObject)
  private
    FPort: string;
    FHost: string;
    FUserName: string;
    FPassword: string;
    FChecking: Boolean;
    FCheckInterval: Integer;
    FMessageBox: Boolean;
    FMessageBeep: Boolean;
    FVisible: Boolean;
  public
    constructor Create;
    destructor Destroy; override;
    property Checking: Boolean read FChecking write FChecking;
    property CheckInterval: Integer read FCheckInterval write FCheckInterval;
    property Host: string read FHost write FHost;
    property MessageBeep: Boolean read FMessageBeep write FMessageBeep;
    property MessageBox: Boolean read FMessageBox write FMessageBox;
    property Password: string read FPassword write FPassword;
    property Port: string read FPort write FPort;
    property UserName: string read FUserName write FUserName;
    property Visible: Boolean read FVisible write FVisible;
  end;

{ TOptionsForm }

  TOptionsForm = class(TForm)
    ServerGroup: TGroupBox;
    HostLabel: TLabel;
    HostEdit: TEdit;
    UserLabel: TLabel;
    UserEdit: TEdit;
    PasswordLabel: TLabel;
    PasswordEdit: TEdit;
    CheckGroup: TGroupBox;
    IntervalCheck: TCheckBox;
    IntervalEdit: TEdit;
    IntervalBar: TScrollBar;
    IntervalLabel: TLabel;
    MessageBoxCheck: TCheckBox;
    MessageBeepCheck: TCheckBox;
    OKButton: TButton;
    CancelButton: TButton;
    procedure IntervalCheckClick(Sender: TObject);
    procedure IntervalBarChange(Sender: TObject);
    procedure IntervalBarScroll(Sender: TObject; ScrollCode: TScrollCode; var ScrollPos: Integer);
    procedure OKButtonClick(Sender: TObject);
  private
    FOptions: TOptions;
  protected
    procedure CheckParams;
    procedure GetParams;
    procedure PutParams;
  public
    function Execute(Options: TOptions): Boolean;
  end;

function ChangeOptions(Options: TOptions): Boolean;

implementation

uses EncryptIt;

{$R *.DFM}

function ChangeOptions(Options: TOptions): Boolean;
var
  OptionsForm: TOptionsForm;
begin
  Result := False;
  OptionsForm := TOptionsForm.Create(Application);
  try
    if OptionsForm.Execute(Options) then Result := True; 
  finally
    OptionsForm.Free;
  end;
end;

{ TOptions }

constructor TOptions.Create;
var
  IniFile: TRegIniFile;
begin
  IniFile := TRegIniFile.Create('Software\Mail Checker\1.0');
  try
    Port := IniFile.ReadString('', 'Port', '110');
    Host := IniFile.ReadString('', 'Host', '');
    UserName := IniFile.ReadString('', 'UserName', '');
    Password := Decrypt(IniFile.ReadString('', 'Password', ''), 3045);
    Checking := IniFile.ReadBool('', 'Checking', True);
    CheckInterval := IniFile.ReadInteger('', 'CheckInterval', 5);
    MessageBox := IniFile.ReadBool('', 'MessageBox', True);
    MessageBeep := IniFile.ReadBool('', 'MessageBeep', True);
    Visible := IniFile.ReadBool('', 'Visible', True);
  finally
    IniFile.Free;
  end;
end;

destructor TOptions.Destroy;
var
  IniFile: TRegIniFile;
begin
  IniFile := TRegIniFile.Create('Software\Mail Checker\1.0');
  try
    IniFile.WriteString('', 'Host', Host);
    IniFile.WriteString('', 'UserName', UserName);
    IniFile.WriteString('', 'Password', Encrypt(Password, 3045));
    IniFile.WriteBool('', 'Checking', Checking);
    IniFile.WriteInteger('', 'CheckInterval', CheckInterval);
    IniFile.WriteBool('', 'MessageBox', MessageBox);
    IniFile.WriteBool('', 'MessageBeep', MessageBeep);
    IniFile.WriteBool('', 'Visible', Visible);
  finally
    IniFile.Free;
  end;
end;

{ TOptionsForm }

procedure TOptionsForm.CheckParams;
var
  I: Integer;
begin
  if IntervalCheck.Checked then
    with IntervalEdit do
    try
      I := StrToIntDef(Text, -1);
      if (I < 0) or (I > 120) then
        raise Exception.Create('      1  120');
    except
      SetFocus;
      raise;
    end;
end;

procedure TOptionsForm.GetParams;
begin
  with FOptions do
  begin
    HostEdit.Text := Host;
    UserEdit.Text := UserName;
    PasswordEdit.Text := Password;
    IntervalCheck.Checked := Checking;
    IntervalEdit.Text := IntToStr(CheckInterval);
    IntervalBar.Position := 120 - CheckInterval;
    MessageBoxCheck.Checked := MessageBox;
    MessageBeepCheck.Checked := MessageBeep;
  end;
end;

procedure TOptionsForm.PutParams;
begin
  with FOptions do
  begin
    Host := HostEdit.Text;
    UserName := UserEdit.Text;
    Password := PasswordEdit.Text;
    Checking := IntervalCheck.Checked;
    CheckInterval := StrToIntDef(IntervalEdit.Text, CheckInterval);
    MessageBox := MessageBoxCheck.Checked;
    MessageBeep := MessageBeepCheck.Checked;
  end;
end;

function TOptionsForm.Execute(Options: TOptions): Boolean;
begin
  FOptions := Options;
  GetParams;
  Result := ShowModal = mrOK;
end;

procedure TOptionsForm.IntervalCheckClick(Sender: TObject);
begin
  if (Sender as TCheckBox).Checked then
  begin
    IntervalEdit.Color := clWindow;
    IntervalEdit.Enabled := True;
    IntervalBar.Enabled := True;
  end else
  begin
    IntervalEdit.Color := clBtnFace;
    IntervalEdit.Enabled := False;
    IntervalBar.Enabled := False;
  end;
end;

procedure TOptionsForm.IntervalBarChange(Sender: TObject);
begin
  IntervalEdit.Text := IntToStr(120 - IntervalBar.Position);
end;

procedure TOptionsForm.IntervalBarScroll(Sender: TObject; ScrollCode: TScrollCode; var ScrollPos: Integer);
begin
  try
    ScrollPos := 120 - StrToInt(IntervalEdit.Text);
    case ScrollCode of
      scLineUp: Dec(ScrollPos, 1);
      scLineDown: Inc(ScrollPos, 1)
    end;
  except
    { Ignore exceptions }
  end;
end;

procedure TOptionsForm.OKButtonClick(Sender: TObject);
begin
  CheckParams;
  PutParams;
  ModalResult := mrOK;
end;

end.
