unit MC_Source;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  StdCtrls, Menus;

type
  TSourceForm = class(TForm)
    PopupMenu: TPopupMenu;
    PopupDecodeItem: TMenuItem;
    PopupBase64Item: TMenuItem;
    PopupQuotedPrintableItem: TMenuItem;
    PopupPlainTextItem: TMenuItem;
    PopupN1Item: TMenuItem;
    PopupCharSetItem: TMenuItem;
    PopupDOSItem: TMenuItem;
    PopupWINItem: TMenuItem;
    PopupKOIItem: TMenuItem;
    PopupN2Item: TMenuItem;
    PopupCopyItem: TMenuItem;
    PopupN3Item: TMenuItem;
    PopupSelectAllItem: TMenuItem;
    SourceMemo: TMemo;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure PopupMenuPopup(Sender: TObject);
    procedure PopupDecodeItemClick(Sender: TObject);
    procedure PopupCharSetItemClick(Sender: TObject);
    procedure PopupCopyItemClick(Sender: TObject);
    procedure PopupSelectAllItemClick(Sender: TObject);
    procedure SourceMemoKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
  private
    FMessage: string;
    FCharSet: Integer;
    FDecode: Integer;
    procedure SetCharSet(Value: Integer);
    procedure SetDecode(Value: Integer);
    procedure SetMessage(Value: string);
    procedure RefreshView;
  end;

procedure ShowMessageSource(const Message: string);

implementation

uses
  MC_Form, MC_Mail, MC_Decode, MC_CharSet;

{$R *.DFM}

procedure ShowMessageSource(const Message: string);
var
  SourceForm: TSourceForm;
begin
  SourceForm := TSourceForm.Create(Application);
  try
    SourceForm.SetCharSet(CheckerForm.CharSet);
    SourceForm.SetMessage(Message);
    SourceForm.Show;
  except
    SourceForm.Free;
    raise;
  end;
end;

{ TSourceForm }

procedure TSourceForm.SetCharSet(Value: Integer);
begin
  FCharSet := Value;
  case Value of
    CS_DOS: PopupDOSItem.Checked := True;
    CS_WIN: PopupWINItem.Checked := True;
    CS_KOI: PopupKOIItem.Checked := True;
  end;
  RefreshView;
end;

procedure TSourceForm.SetDecode(Value: Integer);
begin
  FDecode := Value;
  case Value of
    0: PopupPlainTextItem.Checked := True;
    1: PopupQuotedPrintableItem.Checked := True;
    2: PopupBase64Item.Checked := True;
  end;
  RefreshView;
end;

procedure TSourceForm.SetMessage(Value: string);
begin
  FMessage := Value;
  RefreshView;
end;

procedure TSourceForm.RefreshView;
var
  Header, Body: string;
begin
  Header := ExtractMessageHeaders(FMessage);
  case FDecode of
    0: Body := ExtractMessageBody(FMessage);
    1: Body := DecodeQuotedPrintable(ExtractMessageBody(FMessage));
    2: Body := DecodeBase64(ExtractMessageBody(FMessage));
  end;
  Caption := ChangeCharSet(ExtractMessageHeader(FMessage, SSubject), FCharSet, CS_WIN);
  SourceMemo.Text := ChangeCharSet(Header + #13#10#13#10 + Body, FCharSet, CS_WIN);
end;

procedure TSourceForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  Action := caFree;
end;

procedure TSourceForm.PopupMenuPopup(Sender: TObject);
begin
  PopupCopyItem.Enabled := SourceMemo.SelLength > 0;
end;

procedure TSourceForm.PopupDecodeItemClick(Sender: TObject);
begin
  SetDecode((Sender as TMenuItem).Tag)
end;

procedure TSourceForm.PopupCharSetItemClick(Sender: TObject);
begin
  SetCharSet((Sender as TMenuItem).Tag);
end;

procedure TSourceForm.PopupCopyItemClick(Sender: TObject);
begin
  SourceMemo.Perform(WM_COPY, 0, 0);
end;

procedure TSourceForm.PopupSelectAllItemClick(Sender: TObject);
begin
  SourceMemo.Perform(EM_SETSEL, 0, -1);
end;

procedure TSourceForm.SourceMemoKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if (Shift = [ssShift]) and (Key = VK_F10) then
  begin
    with SourceMemo.ClientOrigin do PopupMenu.Popup(X, Y);
    Key := 0;
  end;
end;

end.

